#pragma once

// [lexeme.h]
// -------------------------------------------------------------------------------------------- //
// lexeme data class
// -------------------------------------------------------------------------------------------- //

namespace scm
{
	class Lexeme {
	public:

		// lexeme types
		// -------------------------------------------------------------------------------------------- //
		enum TYPE {
			IS_INVALID,
			IS_NAME,								// 
			IS_ASSIGNMENT,							// =
			IS_BRACE_OPEN, IS_BRACE_CLOSE,			// {}
			IS_PAREN_OPEN, IS_PAREN_CLOSE,			// ()
			IS_BRACKET_OPEN, IS_BRACKET_CLOSE,		// []
			IS_SEMICOLON,							// ;
			IS_COMMA,								// ,
			IS_INTEGER, IS_FLOAT, IS_BOOL,			// 
			IS_STRING,								//
			IS_OP_ADD, IS_OP_SUB,					// +,-
			IS_OP_MUL, IS_OP_DIV, IS_OP_MOD,		// *,/,%
			IS_OP_PLUS, IS_OP_MINUS,				// +,-
			IS_OP_EXP,								// ^
			IS_OP_EQ, IS_OP_NEQ,					// ==, !=
			IS_OP_LT, IS_OP_GT,						// <, >
			IS_OP_LEQ, IS_OP_GEQ,					// <=, >=
			IS_OP_LOGICAL_AND, IS_OP_LOGICAL_OR,	// &&, ||
			IS_SIN_FUNCTION, IS_COS_FUNCTION,		// __sin, __cos
			IS_TAN_FUNCTION,						// __tan
			IS_LOG_FUNCTION,						// __log
			IS_SQRT_FUNCTION,						// __sqrt
			IS_MIN_FUNCTION, IS_MAX_FUNCTION,		// __min, __max
			IS_VECMIN_FUNCTION, IS_VECMAX_FUNCTION,	// __vecmin, __vecmax
			IS_ABS_FUNCTION,						// __abs
			IS_UNIRAND_FUNCTION,					// __unirand
			IS_DOT_PRODUCT_FUNCTION,				// __dot
			IS_L2NORM_FUNCTION, IS_CNORM_FUNCTION,	// __l2norm, __cnorm
			IS_TO_STRING_FUNCTION,					// __to_string
			IS_SIZE_FUNCTION,						// __size
			IS_DEFINED_FUNCTION,					// __defined
			IS_COMMAND,								// __$
			IS_VECTOR_CTOR,							// __vector
			IS_VECTOR_CONST_CTOR,					// __vector_const
			IS_VECTOR_UNIRAND_CTOR,					// __vector_unirand
			IS_KEY_IF, IS_KEY_THEN,					// 'if', 'then'
			IS_KEY_ELSE, IS_KEY_ENDIF,				// 'else', 'endif'
			IS_KEY_WHILE,							// 'while'
			IS_KEY_DO, IS_KEY_ENDDO					// 'do', 'enddo'
		};
		// -------------------------------------------------------------------------------------------- //

	public:

		// set calls
		// -------------------------------------------------------------------------------------------- //
		void set(const char* _token, const TYPE _type, const int _tag);
		void set(const char* _token, const TYPE _type);
		// -------------------------------------------------------------------------------------------- //

		// get calls
		// -------------------------------------------------------------------------------------------- //
		const char* get_token() const;
		TYPE get_type() const;
		int get_tag() const;
		// -------------------------------------------------------------------------------------------- //

		// check lexeme type
		// -------------------------------------------------------------------------------------------- //
		bool is_operator() const;
		bool is_function() const;
		bool is_ctor() const;
		bool is_value() const;

		static bool is_operator(const Lexeme::TYPE type);
		static bool is_function(const Lexeme::TYPE type);
		static bool is_ctor(const Lexeme::TYPE type);
		static bool is_value(const Lexeme::TYPE type);
		// -------------------------------------------------------------------------------------------- //

		// clear
		// -------------------------------------------------------------------------------------------- //
		void clear();
		// -------------------------------------------------------------------------------------------- //

		// swap & assignment
		// -------------------------------------------------------------------------------------------- //
		const Lexeme& operator=(Lexeme lexeme);
		void swap(Lexeme& lexeme);
		// -------------------------------------------------------------------------------------------- //

		// -------------------------------------------------------------------------------------------- //
		Lexeme();
		Lexeme(const Lexeme& lexeme);
		~Lexeme();
		// -------------------------------------------------------------------------------------------- //

	private:
		// data (private):
		// -------------------------------------------------------------------------------------------- //

		char *token;
		TYPE type;
		int tag;
	};
	// -------------------------------------------------------------------------------------------- //
}
